@@ -659,8 +659,6 @@ eel_format_uri_for_display_internal (const char *uri, gboolean filenames_are_loc
 
 	g_return_val_if_fail (uri != NULL, g_strdup (""));
 
-	g_assert (g_utf8_validate (uri, -1, NULL));
-	
 	canonical_uri = eel_make_uri_canonical (uri);
 
 	/* If there's no fragment and it's a local path. */
@@ -680,6 +678,9 @@ eel_format_uri_for_display_internal (const char *uri, gboolean filenames_are_loc
 		}
 	}
 
+	g_assert (!canonical_uri ||
+		  g_utf8_validate (canonical_uri, -1, NULL));
+
 	g_free (path);
 	return canonical_uri;
 }
@@ -1505,6 +1506,36 @@ eel_is_remote_uri (const char *uri)
 	return !is_local;
 }
 
+/**
+ * eel_is_valid_uri:
+ * @uri: something we wish was a uri.
+ * 
+ *    This is not an exhaustive check, obviously, it
+ * exists to simply catch bad looking URIs before they
+ * go and create duplicates in the lower levels of
+ * nautilus.
+ * 
+ * Return value: FALSE if not valid, TRUE if more clean looking.
+ **/
+gboolean
+eel_is_valid_uri (const char *uri)
+{
+	const guchar *p;
+
+	g_return_val_if_fail (uri != NULL, FALSE);
+
+	if (!has_valid_scheme (uri)) {
+		return FALSE;
+	}
+
+	/* We expect to have a fully valid set of characters */
+	for (p = uri; *p; p++) {
+		if (*p <= 32 || *p >= 128)
+			return FALSE;
+	}
+
+	return TRUE;
+}
 
 GnomeVFSResult
 eel_make_directory_and_parents (GnomeVFSURI *uri, guint permissions)
